from torch.utils import data


class Para2SelfEncWrapper(data.Dataset):
    def __init__(self, ds):
        super(Para2SelfEncWrapper, self).__init__()
        self.ds = ds
        self.o_length = len(ds)

    def __len__(self):
        return 2 * len(self.ds)

    def __getitem__(self, item):
        sid = item % self.o_length
        if item // self.o_length == 0:
            sample = self.ds[sid]['source']
            length = self.ds[sid]['src_len']
        else:
            sample = self.ds[sid]['target']
            length = self.ds[sid]['tgt_len']
        return {
            'source': sample, 'src_len': length,
            'target': sample, 'tgt_len': length
        }


class TextSelfEncWrapper(data.Dataset):
    def __init__(self, ds):
        super(TextSelfEncWrapper, self).__init__()
        self.ds = ds

    def __len__(self):
        return len(self.ds)

    def __getitem__(self, item):
        sample = self.ds[item]['text']
        length = self.ds[item]['length']
        return {
            'source': sample, 'src_len': length,
            'target': sample, 'tgt_len': length
        }


class SynUnsupWrapper(data.Dataset):
    def __init__(self, ds):
        super(SynUnsupWrapper, self).__init__()
        self.ds = ds
        self.o_length = len(ds)

    def __len__(self):
        return 2 * len(self.ds)

    def __getitem__(self, item):
        # 'source': tok_src, 'target': tok_tgt,
        # 'src_len': src_len, 'tgt_len': tgt_len,
        # 'src_h': src_h_seq, 'src_r': src_r_seq, 'src_dep': src_dep,
        # 'tgt_h': tgt_h_seq, 'tgt_r': tgt_r_seq, 'tgt_dep': tgt_dep,

        sid = item % self.o_length
        if item // self.o_length == 0:
            sample = self.ds[sid]['source']
            length = self.ds[sid]['src_len']
            h = self.ds[sid]['src_h']
            r = self.ds[sid]['src_r']
            dep = self.ds[sid]['src_dep']
        else:
            sample = self.ds[sid]['target']
            length = self.ds[sid]['tgt_len']
            h = self.ds[sid]['tgt_h']
            r = self.ds[sid]['tgt_r']
            dep = self.ds[sid]['tgt_dep']
        return {
            'source': sample, 'src_len': length,
            'target': sample, 'tgt_len': length,
            'tgt_h': h, 'tgt_r': r, 'tgt_dep': dep
        }


class RevSimpleWikiTest(data.Dataset):
    def __init__(self, ds):
        super(RevSimpleWikiTest, self).__init__()
        self.ds = ds

    def __len__(self):
        return len(self.ds)

    def __getitem__(self, item):
        # 'source': tok_src, 'target': tok_tgt,
        # 'src_len': src_len, 'tgt_len': tgt_len,
        # 'src_h': src_h_seq, 'src_r': src_r_seq, 'src_dep': src_dep,
        # 'tgt_h': tgt_h_seq, 'tgt_r': tgt_r_seq, 'tgt_dep': tgt_dep,

        return {
            'source': self.ds[item]['target'], 'src_len': self.ds[item]['tgt_len'],
            'target': self.ds[item]['source'], 'tgt_len': self.ds[item]['src_len'],
            'tgt_h': self.ds[item]['src_h'], 'tgt_r': self.ds[item]['src_r'], 'tgt_dep': self.ds[item]['src_dep']}

# class SynParaDataset(data.Dataset):
#     def __init__(self, ds):
#         super(SynParaDataset, self).__init__()
#         self.ds = ds
#
#     def __len__(self):
#         return len(self.ds)
#
#     def __getitem__(self, item):
#         pass
